import { useState, useEffect, useMemo, useRef, createElement, memo, useCallback, forwardRef } from 'react';
import { styled, keyframes, setup } from 'goober';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function _taggedTemplateLiteralLoose(strings, raw) {
  if (!raw) {
    raw = strings.slice(0);
  }

  strings.raw = raw;
  return strings;
}

var genId = /*#__PURE__*/function () {
  var count = 0;
  return function () {
    return (++count).toString();
  };
}();

var TOAST_LIMIT = 20;
var ActionType;

(function (ActionType) {
  ActionType[ActionType["ADD_TOAST"] = 0] = "ADD_TOAST";
  ActionType[ActionType["UPDATE_TOAST"] = 1] = "UPDATE_TOAST";
  ActionType[ActionType["UPSERT_TOAST"] = 2] = "UPSERT_TOAST";
  ActionType[ActionType["DISMISS_TOAST"] = 3] = "DISMISS_TOAST";
  ActionType[ActionType["REMOVE_TOAST"] = 4] = "REMOVE_TOAST";
  ActionType[ActionType["TOAST"] = 5] = "TOAST";
  ActionType[ActionType["ADD_PAUSE"] = 6] = "ADD_PAUSE";
})(ActionType || (ActionType = {}));

var reducer = function reducer(queue, action) {
  switch (action.type) {
    case ActionType.ADD_TOAST:
      return [action.toast].concat(queue).slice(0, TOAST_LIMIT);

    case ActionType.UPDATE_TOAST:
      return queue.map(function (t) {
        return t.id === action.toast.id ? _extends({}, t, action.toast) : t;
      });

    case ActionType.UPSERT_TOAST:
      var toast = action.toast;
      return queue.find(function (t) {
        return t.id === toast.id;
      }) ? reducer(queue, {
        type: ActionType.UPDATE_TOAST,
        toast: toast
      }) : reducer(queue, {
        type: ActionType.ADD_TOAST,
        toast: toast
      });

    case ActionType.DISMISS_TOAST:
      return queue.map(function (t) {
        return t.id === action.toastId || action.toastId === undefined ? _extends({}, t, {
          visible: false
        }) : t;
      });

    case ActionType.REMOVE_TOAST:
      if (action.toastId === undefined) {
        return [];
      }

      return queue.filter(function (t) {
        return t.id !== action.toastId;
      });

    case ActionType.ADD_PAUSE:
      return queue.map(function (t) {
        return _extends({}, t, {
          duration: t.duration + action.duration
        });
      });
  }
};
var listeners = [];
var memoryQueue = [];
var dispatch = function dispatch(action) {
  memoryQueue = reducer(memoryQueue, action);
  listeners.forEach(function (listener) {
    listener(action);
  });
};
var useStore = function useStore() {
  var _useState = useState(memoryQueue),
      queue = _useState[0],
      setQueue = _useState[1];

  useEffect(function () {
    // In case multiple notifications are submitted in sync
    var actionQueue = [];

    var handler = function handler(action) {
      actionQueue.push(action);
      setQueue(actionQueue.reduce(reducer, queue));
    };

    listeners.push(handler);
    return function () {
      var index = listeners.indexOf(handler);

      if (index > -1) {
        listeners.splice(index, 1);
      }
    };
  }, [queue]);
  return queue;
};

var defaultTimeouts = /*#__PURE__*/new Map([['blank', 4000], ['error', 4000], ['success', 2000], ['loading', 30000]]);

var createToast = function createToast(message, type, opts) {
  if (type === void 0) {
    type = 'blank';
  }

  return _extends({
    id: (opts == null ? void 0 : opts.id) || genId(),
    createdAt: Date.now(),
    visible: true,
    type: type,
    role: 'status',
    ariaLive: 'polite',
    duration: defaultTimeouts.get(type) || 4000,
    message: message
  }, opts);
};

var createHandler = function createHandler(type) {
  return function (message, options) {
    var toast = createToast(message, type, options);
    dispatch({
      type: ActionType.UPSERT_TOAST,
      toast: toast
    });
    return toast.id;
  };
};

var toast = function toast(message, opts) {
  return createHandler('blank')(message, opts);
};

toast.error = /*#__PURE__*/createHandler('error');
toast.success = /*#__PURE__*/createHandler('success');
toast.loading = /*#__PURE__*/createHandler('loading');

toast.dismiss = function (toastId) {
  return dispatch({
    type: ActionType.DISMISS_TOAST,
    toastId: toastId
  });
};

toast.remove = function (toastId) {
  return dispatch({
    type: ActionType.REMOVE_TOAST,
    toastId: toastId
  });
};

toast.promise = function (promise, msgs, opts) {
  var id = toast.loading(msgs.loading, opts == null ? void 0 : opts.loading);
  promise.then(function (p) {
    toast.success(typeof msgs.success === 'function' ? msgs.success(p) : msgs.success, _extends({
      id: id
    }, opts == null ? void 0 : opts.success));
    return p;
  })["catch"](function (e) {
    toast.error(typeof msgs.error === 'function' ? msgs.error(e) : msgs.error, _extends({
      id: id
    }, opts == null ? void 0 : opts.error));
  });
  return promise;
};

var useToaster = function useToaster() {
  var toasts = useStore();
  var visibleToasts = toasts.filter(function (t) {
    return t.visible;
  });

  var _useState = useState(false),
      pauseAt = _useState[0],
      setPausedAt = _useState[1];

  useEffect(function () {
    if (pauseAt) {
      return;
    }

    var now = Date.now();
    var timeouts = toasts.map(function (t) {
      var duration = t.duration - (now - t.createdAt);

      var dismiss = function dismiss() {
        dispatch({
          type: ActionType.DISMISS_TOAST,
          toastId: t.id
        });
        setTimeout(function () {
          dispatch({
            type: ActionType.REMOVE_TOAST,
            toastId: t.id
          });
        }, 1000);
      };

      if (duration < 0) {
        if (t.visible) {
          dismiss();
        }

        return;
      }

      return setTimeout(dismiss, duration);
    });
    return function () {
      timeouts.forEach(function (timeout) {
        return timeout && clearTimeout(timeout);
      });
    };
  }, [toasts, pauseAt]);
  var handlers = useMemo(function () {
    return {
      startPause: function startPause() {
        setPausedAt(Date.now());
      },
      endPause: function endPause() {
        if (pauseAt) {
          var diff = Date.now() - pauseAt;
          dispatch({
            type: ActionType.ADD_PAUSE,
            duration: diff
          });
          setPausedAt(false);
        }
      },
      updateHeight: function updateHeight(toastId, height) {
        return dispatch({
          type: ActionType.UPDATE_TOAST,
          toast: {
            id: toastId,
            height: height
          }
        });
      },
      calculateOffset: function calculateOffset(toastId, _ref) {
        var _ref$reverseOrder = _ref.reverseOrder,
            reverseOrder = _ref$reverseOrder === void 0 ? false : _ref$reverseOrder,
            _ref$margin = _ref.margin,
            margin = _ref$margin === void 0 ? 8 : _ref$margin;
        var index = visibleToasts.findIndex(function (toast) {
          return toast.id === toastId;
        });
        var offset = index !== -1 ? visibleToasts.slice.apply(visibleToasts, reverseOrder ? [index + 1] : [0, index]).reduce(function (acc, t) {
          return acc + (t.height || 0) + margin;
        }, 0) : 0;
        return offset;
      }
    };
  }, [toasts, visibleToasts, pauseAt]);
  return {
    toasts: toasts,
    visibleToasts: visibleToasts,
    handlers: handlers
  };
};

var usePreserve = function usePreserve(value) {
  var ref = useRef();
  var preservedValue = value || ref.current;

  if (value) {
    ref.current = value;
  }

  return preservedValue;
};

function _templateObject4() {
  var data = _taggedTemplateLiteralLoose(["\n  width: 20px;\n  opacity: 0;\n  height: 20px;\n  border-radius: 10px;\n  background: ", ";\n  position: relative;\n  transform: rotate(45deg);\n\n  animation: ", " 0.3s cubic-bezier(0.175, 0.885, 0.32, 1.275)\n    forwards;\n  animation-delay: 100ms;\n\n  &:after,\n  &:before {\n    content: '';\n    animation: ", " 0.15s ease-out forwards;\n    animation-delay: 150ms;\n    position: absolute;\n    border-radius: 3px;\n    opacity: 0;\n    background: ", ";\n    bottom: 9px;\n    left: 4px;\n    height: 2px;\n    width: 12px;\n  }\n\n  &:before {\n    animation: ", " 0.15s ease-out forwards;\n    animation-delay: 180ms;\n    transform: rotate(90deg);\n  }\n"]);

  _templateObject4 = function _templateObject4() {
    return data;
  };

  return data;
}

function _templateObject3() {
  var data = _taggedTemplateLiteralLoose(["\n  from {\n    transform: scale(0) rotate(90deg);\n\t opacity: 0;\n  }\n\n  to {\n    transform: scale(1) rotate(90deg);\n\t opacity: 1;\n  }\n"]);

  _templateObject3 = function _templateObject3() {
    return data;
  };

  return data;
}

function _templateObject2() {
  var data = _taggedTemplateLiteralLoose(["\n  from {\n    transform: scale(0);\n\t opacity: 0;\n  }\n\n  to {\n    transform: scale(1);\n\t opacity: 1;\n  }\n"]);

  _templateObject2 = function _templateObject2() {
    return data;
  };

  return data;
}

function _templateObject() {
  var data = _taggedTemplateLiteralLoose(["\n  from {\n    transform: scale(0) rotate(45deg);\n\t opacity: 0;\n  }\n\n  to {\n    transform: scale(1) rotate(45deg);\n\t opacity: 1;\n  }\n"]);

  _templateObject = function _templateObject() {
    return data;
  };

  return data;
}
var circleAnimation = /*#__PURE__*/keyframes( /*#__PURE__*/_templateObject());
var firstLineAnimation = /*#__PURE__*/keyframes( /*#__PURE__*/_templateObject2());
var secondLineAnimation = /*#__PURE__*/keyframes( /*#__PURE__*/_templateObject3());
var ErrorIcon = /*#__PURE__*/styled('div')( /*#__PURE__*/_templateObject4(), function (p) {
  return p.primary || '#ff4b4b';
}, circleAnimation, firstLineAnimation, function (p) {
  return p.secondary || '#fff';
}, secondLineAnimation);

function _templateObject2$1() {
  var data = _taggedTemplateLiteralLoose(["\n  width: 12px;\n  height: 12px;\n  box-sizing: border-box;\n  border: 2px solid;\n  border-radius: 100%;\n  border-color: ", ";\n  border-right-color: ", ";\n  animation: ", " 1s linear infinite;\n"]);

  _templateObject2$1 = function _templateObject2() {
    return data;
  };

  return data;
}

function _templateObject$1() {
  var data = _taggedTemplateLiteralLoose(["\n  from {\n    transform: rotate(0deg);\n  }\n  to {\n    transform: rotate(360deg);\n  }\n"]);

  _templateObject$1 = function _templateObject() {
    return data;
  };

  return data;
}
var rotate = /*#__PURE__*/keyframes( /*#__PURE__*/_templateObject$1());
var LoaderIcon = /*#__PURE__*/styled('div')( /*#__PURE__*/_templateObject2$1(), function (p) {
  return p.secondary || '#e0e0e0';
}, function (p) {
  return p.primary || '#616161';
}, rotate);

function _templateObject3$1() {
  var data = _taggedTemplateLiteralLoose(["\n  width: 20px;\n  opacity: 0;\n  height: 20px;\n  border-radius: 10px;\n  background: ", ";\n  position: relative;\n  transform: rotate(45deg);\n\n  animation: ", " 0.3s cubic-bezier(0.175, 0.885, 0.32, 1.275)\n    forwards;\n  animation-delay: 100ms;\n  &:after {\n    content: '';\n    box-sizing: border-box;\n    animation: ", " 0.2s ease-out forwards;\n    opacity: 0;\n    animation-delay: 200ms;\n    position: absolute;\n    border-right: 2px solid;\n    border-bottom: 2px solid;\n    border-color: ", ";\n    bottom: 6px;\n    left: 6px;\n    height: 10px;\n    width: 6px;\n  }\n"]);

  _templateObject3$1 = function _templateObject3() {
    return data;
  };

  return data;
}

function _templateObject2$2() {
  var data = _taggedTemplateLiteralLoose(["\n  0% {\n\t\theight: 0;\n\t\twidth: 0;\n\t\topacity: 0;\n  }\n\n  40% {\n\t\theight: 0;\n\t\twidth: 6px;\n\t\topacity: 1;\n  }\n\n  100% {\n    opacity: 1;\n\t\theight: 10px;\n  }\n"]);

  _templateObject2$2 = function _templateObject2() {
    return data;
  };

  return data;
}

function _templateObject$2() {
  var data = _taggedTemplateLiteralLoose(["\n  from {\n    transform: scale(0) rotate(45deg);\n\t opacity: 0;\n  }\n\n  to {\n    transform: scale(1) rotate(45deg);\n\t opacity: 1;\n  }\n"]);

  _templateObject$2 = function _templateObject() {
    return data;
  };

  return data;
}
var circleAnimation$1 = /*#__PURE__*/keyframes( /*#__PURE__*/_templateObject$2());
var checkmarkAnimation = /*#__PURE__*/keyframes( /*#__PURE__*/_templateObject2$2());
var CheckmarkIcon = /*#__PURE__*/styled('div')( /*#__PURE__*/_templateObject3$1(), function (p) {
  return p.primary || '#61d345';
}, circleAnimation$1, checkmarkAnimation, function (p) {
  return p.secondary || '#fff';
});

function _templateObject2$3() {
  var data = _taggedTemplateLiteralLoose(["\n  position: relative;\n  display: flex;\n  justify-content: center;\n  align-items: center;\n  width: 20px;\n  height: 20px;\n"]);

  _templateObject2$3 = function _templateObject2() {
    return data;
  };

  return data;
}

function _templateObject$3() {
  var data = _taggedTemplateLiteralLoose(["\n  position: absolute;\n"]);

  _templateObject$3 = function _templateObject() {
    return data;
  };

  return data;
}
var StatusWrapper = /*#__PURE__*/styled('div')( /*#__PURE__*/_templateObject$3());
var IndicatorWrapper = /*#__PURE__*/styled('div')( /*#__PURE__*/_templateObject2$3());
var Indicator = function Indicator(_ref) {
  var type = _ref.type,
      theme = _ref.theme;

  if (type === 'blank') {
    return null;
  }

  return createElement(IndicatorWrapper, null, createElement(LoaderIcon, Object.assign({}, theme == null ? void 0 : theme.success)), type !== 'loading' && createElement(StatusWrapper, null, type === 'error' ? createElement(ErrorIcon, Object.assign({}, theme == null ? void 0 : theme.error)) : createElement(CheckmarkIcon, Object.assign({}, theme == null ? void 0 : theme.success))));
};

function _templateObject2$4() {
  var data = _taggedTemplateLiteralLoose(["\n  position: relative;\n  transform: scale(0.6);\n  opacity: 0.4;\n  min-width: 20px;\n  animation: ", " 0.3s 0.12s cubic-bezier(0.175, 0.885, 0.32, 1.275)\n    forwards;\n"]);

  _templateObject2$4 = function _templateObject2() {
    return data;
  };

  return data;
}

function _templateObject$4() {
  var data = _taggedTemplateLiteralLoose(["\nfrom {\n  transform: scale(0.6);\n  opacity: 0.4;\n}\n\nto {\n  transform: scale(1);\n  opacity: 1;\n}\n"]);

  _templateObject$4 = function _templateObject() {
    return data;
  };

  return data;
}
var enter = /*#__PURE__*/keyframes( /*#__PURE__*/_templateObject$4());
var AnimatedIconWrapper = /*#__PURE__*/styled('div')( /*#__PURE__*/_templateObject2$4(), enter);

function _templateObject4$1() {
  var data = _taggedTemplateLiteralLoose(["", ""]);

  _templateObject4$1 = function _templateObject4() {
    return data;
  };

  return data;
}

function _templateObject3$2() {
  var data = _taggedTemplateLiteralLoose(["", ""]);

  _templateObject3$2 = function _templateObject3() {
    return data;
  };

  return data;
}

function _templateObject2$5() {
  var data = _taggedTemplateLiteralLoose(["\n  margin: 4px 10px;\n  color: inherit;\n  flex: 1;\n"]);

  _templateObject2$5 = function _templateObject2() {
    return data;
  };

  return data;
}

function _templateObject$5() {
  var data = _taggedTemplateLiteralLoose(["\n  display: flex;\n  align-items: center;\n  background: #fff;\n  color: #363636;\n  text-align: center;\n  line-height: 1.3;\n  box-shadow: 0 3px 10px rgba(0, 0, 0, 0.1), 0 3px 3px rgba(0, 0, 0, 0.05);\n  max-width: 300px;\n  margin: 8px;\n  pointer-events: auto;\n  padding: 6px 8px;\n  border-radius: 8px;\n"]);

  _templateObject$5 = function _templateObject() {
    return data;
  };

  return data;
}

var enterAnimation = function enterAnimation(reverse) {
  return "\n0% {transform: translate3d(0," + (reverse ? -1 : 1) * -80 + "px,0) scale(.6); opacity:.5;}\n100% {transform: translate3d(0,0,0) scale(1); opacity:1;}\n";
};

var exitAnimation = function exitAnimation(reverse) {
  return "\n0% {transform: translate3d(0,0,0) scale(1); opacity:1;}\n100% {transform: translate3d(0," + (reverse ? -1 : 1) * -130 + "px, 0) scale(.5); opacity:0;}\n";
};

var ToastBarBase = /*#__PURE__*/styled('div', forwardRef)( /*#__PURE__*/_templateObject$5());
var Message = /*#__PURE__*/styled('p')( /*#__PURE__*/_templateObject2$5());
var ToastBar = /*#__PURE__*/memo(function (_ref) {
  var toast = _ref.toast,
      position = _ref.position,
      props = _objectWithoutPropertiesLoose(_ref, ["toast", "position"]);

  var prevToast = usePreserve(toast);
  var ref = useCallback(function (el) {
    if (el) {
      var boundingRect = el.getBoundingClientRect();
      props.onHeight(boundingRect.height);
    }
  }, []);

  if (!prevToast) {
    return null;
  }

  var top = position.includes('top');
  var verticalStyle = top ? {
    top: 0
  } : {
    bottom: 0
  };
  var horizontalStyle = position.includes('left') ? {
    left: 0
  } : position.includes('right') ? {
    right: 0
  } : {
    left: 0,
    pointerEvents: 'none',
    right: 0,
    justifyContent: 'center'
  };
  var animationStyle = prevToast != null && prevToast.height ? prevToast.visible ? {
    animation: keyframes(_templateObject3$2(), enterAnimation(!top)) + " 0.35s cubic-bezier(.21,1.02,.73,1) forwards"
  } : {
    animation: keyframes(_templateObject4$1(), exitAnimation(!top)) + " 0.8s forwards cubic-bezier(.06,.71,.55,1)",
    pointerEvents: 'none'
  } : {
    opacity: 0
  };

  var renderIcon = function renderIcon() {
    var icon = prevToast.icon;

    if (icon !== undefined) {
      if (typeof icon === 'string') {
        return createElement(AnimatedIconWrapper, null, icon);
      } else {
        return createElement(IndicatorWrapper, null, icon);
      }
    }

    return createElement(Indicator, {
      theme: _extends({}, props.iconTheme, prevToast.iconTheme),
      type: prevToast.type
    });
  };

  var style = _extends({}, animationStyle, props.style, toast.style);

  return createElement("div", {
    style: _extends({
      position: 'fixed',
      display: 'flex'
    }, horizontalStyle, verticalStyle, {
      transition: 'all 230ms cubic-bezier(.21,1.02,.73,1)',
      transform: "translateY(" + props.offset * (top ? 1 : -1) + "px)",
      zIndex: toast.visible ? style.zIndex || 9999 : undefined
    })
  }, createElement(ToastBarBase, {
    ref: ref,
    className: props.className,
    style: style
  }, renderIcon(), createElement(Message, {
    role: prevToast.role,
    "aria-live": prevToast.ariaLive
  }, prevToast.message)));
});

setup(createElement);
var Toaster = function Toaster(_ref) {
  var _style;

  var reverseOrder = _ref.reverseOrder,
      _ref$position = _ref.position,
      position = _ref$position === void 0 ? 'top-center' : _ref$position,
      iconTheme = _ref.iconTheme,
      props = _objectWithoutPropertiesLoose(_ref, ["reverseOrder", "position", "iconTheme"]);

  var _useToaster = useToaster(),
      toasts = _useToaster.toasts,
      handlers = _useToaster.handlers;

  return createElement("div", {
    style: (_style = {
      position: 'fixed'
    }, _style[position.includes('top') ? 'top' : 'bottom'] = 0, _style),
    onMouseEnter: handlers.startPause,
    onMouseLeave: handlers.endPause
  }, toasts.map(function (t) {
    return createElement(ToastBar, {
      key: t.id,
      onHeight: function onHeight(height) {
        return handlers.updateHeight(t.id, height);
      },
      toast: t,
      position: position,
      offset: handlers.calculateOffset(t.id, {
        reverseOrder: reverseOrder
      }),
      iconTheme: iconTheme,
      className: props.toastClassName,
      style: props.toastStyle
    });
  }));
};

export default toast;
export { ActionType, CheckmarkIcon, ErrorIcon, LoaderIcon, ToastBar, Toaster, dispatch, toast, useToaster };
//# sourceMappingURL=react-hot-toast.esm.js.map
